# ----------------------------------------------------------------------
#  APP: MD Showcase
#
#  This is the main app for a trajectory viewer used to show off VMD
#  trajectories.
# ======================================================================
#  Copyright (c) 2013  Michael McLennan
# ======================================================================
wm title . "MD Showcase"
wm geometry . 800x600
#wm protocol . WM_DELETE_WINDOW {Rappture::Logger::cleanup; exit}
wm protocol . WM_DELETE_WINDOW exit

set ToolDir [file dirname [file normalize [info script]]]
set InstallDir [file dirname $ToolDir]
lappend auto_path $ToolDir

package require Rappture
package require RapptureGUI
package require Img
package require Itk
package require base64
Rappture::icon::searchpath [file join $InstallDir images]

option add *Entry.background white startupFile
option add *TreeView.font {Helvetica -12} startupFile
option add *Controlbox.entryBackground #eee startupFile
option add *Controlbox.entryForeground black startupFile
option add *cntl*Button.relief flat startupFile
option add *cntl*Button.overRelief raised startupFile
option add *cntl*font {Helvetica -12 italic} startupFile
option add *cntl*VCR.font {Helvetica -10} startupFile
option add *nav*Button.relief flat startupFile
option add *nav*Button.overRelief raised startupFile
option add *nav*Button.anchor w startupFile
option add *nav*Button.padX 2 startupFile
option add *nav*Button.font {Helvetica -12 italic} startupFile
option add *nav.list.width 3i startupFile
option add *download*Button.relief flat startupFile
option add *download*Button.overRelief raised startupFile
option add *download*font {Helvetica -12} startupFile
option add *upload*info.borderWidth 2 startupFile
option add *upload*info.relief flat startupFile
option add *upload*info.background #666 startupFile
option add *upload*info.text.background white startupFile
option add *upload*info.text.font {Helvetica -12} startupFile
option add *upload*info.text.borderWidth 8 startupFile
option add *upload*info.text.relief flat startupFile
option add *upload*info*popup*Label.font {Helvetica -14} startupFile
option add *upload*cntls*Frame.background #666 startupFile
option add *upload*cntls*highlightBackground #666 startupFile
option add *citations.l.font {Helvetica -18 bold} startupFile
option add *citations*text.relief flat startupFile
option add *citations*text.font {Courier -12} startupFile

# plug in Rappture's error handler
Rappture::bugreport::install

# use Rappture's grab stack
Rappture::grab::init

# load resource settings so we know what rendering server to talk to
VMDvisViewer_init_resources
Rappture::resources::load

# set up for file transfer
Rappture::filexfer::init

# see if this user belongs to the list of authorized editors
set Editor 0
if {[catch {exec groups} result] == 0} {
    if {[regexp {(^| )mdshowcasebeta($| )} $result]} {
        set Editor 1
    }
}

# look for shared data in this folder
set DataFolder /data/tools/mdshowcase

# set up keyboard shortcuts
bind kbshortcut <KeyPress-r> {main_scene_mode rotate}
bind kbshortcut <KeyPress-R> {main_scene_mode rotate}
bind kbshortcut <KeyPress-t> {main_scene_mode pan}
bind kbshortcut <KeyPress-T> {main_scene_mode pan}
bind kbshortcut <KeyPress-s> {main_scene_mode zoom}
bind kbshortcut <KeyPress-S> {main_scene_mode zoom}

bind kbshortcut <Control-KeyPress-r> {main_scene_reset}

bind kbshortcut <KeyPress-j> {$Widgets(viewer) rotate x 2}
bind kbshortcut <Control-KeyPress-n> {$Widgets(viewer) rotate x 2}
bind kbshortcut <KeyPress-k> {$Widgets(viewer) rotate x -2}
bind kbshortcut <Control-KeyPress-p> {$Widgets(viewer) rotate x -2}
bind kbshortcut <KeyPress-l> {$Widgets(viewer) rotate y 2}
bind kbshortcut <Control-KeyPress-f> {$Widgets(viewer) rotate y 2}
bind kbshortcut <KeyPress-h> {$Widgets(viewer) rotate y -2}
bind kbshortcut <Control-KeyPress-b> {$Widgets(viewer) rotate y -2}
bind kbshortcut <KeyPress-g> {$Widgets(viewer) rotate z 2}
bind kbshortcut <KeyPress-G> {$Widgets(viewer) rotate z -2}

bind kbshortcut <Control-KeyPress-a> {$Widgets(viewer) zoom in 1.1}
bind kbshortcut <Control-KeyPress-z> {$Widgets(viewer) zoom in 0.9}

bind kbshortcut <KeyPress-f> {$Widgets(viewer) play step>}
bind kbshortcut <KeyPress-F> {$Widgets(viewer) play step>}
bind kbshortcut <KeyPress-plus> {$Widgets(viewer) play step>}
bind kbshortcut <KeyPress-b> {$Widgets(viewer) play <step}
bind kbshortcut <KeyPress-B> {$Widgets(viewer) play <step}
bind kbshortcut <KeyPress-minus> {$Widgets(viewer) play <step}
bind kbshortcut <KeyPress-period> {$Widgets(VCR) isplaying 1}
bind kbshortcut <KeyPress-greater> {$Widgets(VCR) isplaying 1}
bind kbshortcut <KeyPress-comma> {$Widgets(VCR) isplaying -1}
bind kbshortcut <KeyPress-less> {$Widgets(VCR) isplaying -1}
bind kbshortcut <KeyPress-slash> {$Widgets(VCR) isplaying 0}
bind kbshortcut <KeyPress-question> {$Widgets(VCR) isplaying 0}

set btags [bindtags .]
bindtags . [linsert $btags 0 kbshortcut]

# if the $RAPPTURE_LOG directory is set to a directory used for
# logging, then open a log file and start logging
#Rappture::Logger::init

# ----------------------------------------------------------------------
# PARSER FOR CONFIGURATION FILES
# ----------------------------------------------------------------------
array set Showcase {
    file ""
    scenes ""
    citations ""
}

set MainParser [Rappture::Parser ::#auto]
$MainParser define scene {name body} {
    global Showcase
    lappend Showcase(scenes) [ShowcaseScene ::#auto $body -name $name]
}

$MainParser define citation {body} {
    global Showcase
    lappend Showcase(citations) [Citation ::#auto $body]
}

# ----------------------------------------------------------------------
# USAGE: main_load <shoFile> ?<sceneNum>?
#
# Used to load the configuration file into this program.  If the
# optional <sceneNum> is specified, then it indicates which scene should
# be selected first.  If there is no <sceneNum>, then the top scene (0)
# will be selected.
# ----------------------------------------------------------------------
proc main_load {shoFile {sceneNum ""}} {
    global Widgets Showcase MainParser
    $Widgets(viewer) play pause

    # couldn't get a valid input file? then fail
    if {![file readable $shoFile]} {
        tk_messageBox -icon error -message "Can't read configuration file \"$shoFile\""
        return
    }

    # clear any old content
    $Widgets(viewer) scene forget
    $Widgets(scenelist) delete all
    foreach obj $Showcase(scenes) {
       itcl::delete object $obj
    }
    set Showcase(scenes) ""
    foreach obj $Showcase(citations) {
       itcl::delete object $obj
    }
    set Showcase(citations) ""
    set Showcase(file) ""

    if {[catch {
           set fid [open $shoFile r]
           set info [read $fid]
           close $fid
           $MainParser load $info
        } result]} {
        tk_messageBox -title "MD Showcase Error" -icon error -message "Error loading file \"$shoFile\":\n$result"
        return
    }

    set Showcase(file) $shoFile
    foreach obj $Showcase(scenes) {
        if {[llength [$obj cget -files]] == 0} {
            tk_messageBox -title "MD Showcase Error" -icon error -message "No trajectory files specified for scene \"[$obj cget -name]\" in $shoFile"
        }

        $Widgets(scenelist) insert end -text [$obj cget -name] \
            -desc [$obj cget -description] -data $obj

        $Widgets(viewer) scene define \
            -name [$obj cget -name] \
            -files [$obj cget -files] \
            -vmdscript [$obj cget -vmdscript] \
            -frames [$obj cget -frames] \
            -play [$obj cget -play] \
            -viewscript [$obj cget -viewscript]
    }

    if {![string is int $sceneNum] || $sceneNum < 0} {
        set sceneNum 0
    }
    if {$sceneNum >= [llength $Showcase(scenes)]} {
        set sceneNum [expr {[llength $Showcase(scenes)]-1}]
    }
    $Widgets(main).nav.list select $sceneNum

    .main current main
}

# ----------------------------------------------------------------------
# USAGE: main_scene_change <sceneObj>
#
# Invoked whenever a new scene is selected to install the scene data
# into the VMD viewer.
# ----------------------------------------------------------------------
proc main_scene_change {sceneObj} {
    global Widgets

    # catch the state of the speed/smooth controls
    set curval [$Widgets(VCR) speed -current]
    set setval [$Widgets(VCR) speed -preset]
    set speedPreset [expr {$setval eq "--" || abs($curval-$setval) < 0.01}]

    set curval [$Widgets(VCR) smooth -current]
    set setval [$Widgets(VCR) smooth -preset]
    set smoothPreset [expr {$setval eq "--" || abs($curval-$setval) < 0.01}]

    # pause any animation and show the new scene
    set vname [$sceneObj cget -name]
    $Widgets(viewer) play pause
    $Widgets(viewer) scene show $vname

    # set the color of the selected scene to match the background
    array set opts [$Widgets(viewer) scene vmdinfo $vname]
    set bg black
    set fg white
    set dimbg DimGray
    set dimfg white
    if {!$opts(backgroundgradient)} {
        set bg $opts(color-Background)
        array set VMDcolors $opts(colorvals)
        if {[info exists VMDcolors($bg)]} {
            # if we have the exact color definition from VMD, then use that.
            # VMD's defintion of "gray" and "blue2" is a little different
            # from the usual X convention.
            set bg $VMDcolors($bg)
        }

        foreach {h s v} [Rappture::color::RGBtoHSV $bg] break
        set fg [expr {($v < 0.5) ? "white" : "black"}]

        if {abs($v-0.5) < 0.1 && $s < 0.2} {
            # gray background -- choose another color for dim choices
            set dimbg #9999ff
            set dimfg black
        }
    }
    $Widgets(scenelist) configure \
        -entrybackground $dimbg -entryforeground $dimfg \
        -selectedentrybackground $bg -selectedentryforeground $fg

    # if this scene has a range of frames, highlight in the VCR controls
    set range [$sceneObj cget -frames]
    if {[llength $range] == 2} {
        $Widgets(VCR) configure -range $range
    } else {
        $Widgets(VCR) configure -range ""
    }

    # transfer settings for speed and smoothness
    set val [$sceneObj cget -speed]
    if {$val eq ""} {
        $Widgets(VCR) speed -preset --
    } else {
        $Widgets(VCR) speed -preset $val
        if {$speedPreset} {
            $Widgets(VCR) speed -current $val
        }
    }

    set val [$sceneObj cget -smooth]
    if {$val eq ""} {
        $Widgets(VCR) smooth -preset --
    } else {
        $Widgets(VCR) smooth -preset $val
        if {$smoothPreset} {
            $Widgets(VCR) smooth -current $val
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_scene_reset
#
# Invoked whenever a new scene is selected to install the scene data
# into the VMD viewer.
# ----------------------------------------------------------------------
proc main_scene_reset {} {
    global Widgets
    $Widgets(viewer) reset
}

# ----------------------------------------------------------------------
# USAGE: main_scene_mode rotate|pan|zoom
#
# Sets the controls for the mouse on the view to one of three modes
# of operation:  rotate, pan, or zoom.
# ----------------------------------------------------------------------
proc main_scene_mode {mode} {
    global Widgets

    set allowed {rotate zoom pan}
    if {[lsearch $allowed $mode] < 0} {
        error "bad mode \"$mode\": should be [join $allowed {, }]"
    }

    # turn off all buttons
    foreach widget $allowed {
        set imh [Rappture::icon ${widget}0]
        $Widgets(main).view.cntl.$widget configure -image $imh
    }

    # turn on current mode
    set imh [Rappture::icon ${mode}1]
    $Widgets(main).view.cntl.$mode configure -image $imh

    # set the corresponding view mode
    $Widgets(viewer) play pause
    $Widgets(viewer) mouseMode $mode
}

# ----------------------------------------------------------------------
# USAGE: main_scene_frames position <frameNum>
# USAGE: main_scene_frames play>>
# USAGE: main_scene_frames <<play
# USAGE: main_scene_frames pause
# USAGE: main_scene_frames mode <value>
# USAGE: main_scene_frames speed <value>
# USAGE: main_scene_frames smooth <value>
#
# Invoked when the user interacts with the VCR controls that affect
# time frames for the trajectory.  Starts/stops playback and affects
# speed or jumps to a particular frame.
# ----------------------------------------------------------------------
proc main_scene_frames {option args} {
    global Widgets
    switch -- $option {
        position {
            set num [lindex $args 0]
            $Widgets(viewer) play goto $num
        }
        play>> - <<play - pause {
            $Widgets(viewer) play $option
        }
        mode {
            set newmode [lindex $args 0]
            $Widgets(viewer) play mode $newmode
        }
        speed {
            set newspeed [lindex $args 0]
            $Widgets(viewer) play speed $newspeed
        }
        smooth {
            set newsmooth [lindex $args 0]
            $Widgets(viewer) play smooth $newsmooth
        }
        default {
            error "bad operation \"$option\": should be one of position, play>>, <<play, pause, mode, speed"
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_scene_changes at <num>
# USAGE: main_scene_changes goto <num>
# USAGE: main_scene_changes pause
# USAGE: main_scene_changes playmode <mode> <dir>
# USAGE: main_scene_changes maxframes <num>
# USAGE: main_scene_changes waiting <boolean>
#
# Invoked when the user interacts with the VCR controls that affect
# time frames for the trajectory.  Starts/stops playback and affects
# speed or jumps to a particular frame.
# ----------------------------------------------------------------------
proc main_scene_changes {option args} {
    global Widgets
    switch -- $option {
        at {
            set num [lindex $args 0]
            $Widgets(VCR) progress $num -nocommand
        }
        goto {
            set num [lindex $args 0]
            $Widgets(VCR) progress $num
        }
        pause {
            $Widgets(VCR) isplaying 0
        }
        playmode {
            set newmode [lindex $args 0]
            $Widgets(VCR) playmode $newmode

            set dir [lindex $args 1]
            set dir [expr {($dir eq "backward") ? -1 : 1}]
            $Widgets(VCR) isplaying $dir
        }
        maxframes {
            set num [lindex $args 0]
            $Widgets(VCR) configure -max $num
        }
        waiting {
            set state [lindex $args 0]
            if {$state} {
                blt::busy hold .
            } else {
                blt::busy release .
            }
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_citations popup
# USAGE: main_citations get text|bibtex|endnote
# USAGE: main_citations download text|bibtex|endnote
# USAGE: main_citations cancel
#
# Returns some text in the specified format for the citations associated
# with the current showcase.
# ----------------------------------------------------------------------
proc main_citations {option args} {
    global Showcase Widgets

    switch -- $option {
        popup {
            $Widgets(citations-info) configure -state normal
            $Widgets(citations-info) delete 1.0 end
            $Widgets(citations-info) insert end [main_citations get text]
            $Widgets(citations-info) configure -state disabled

            $Widgets(citations) activate $Widgets(main).nav.cite right
        }
        cancel {
            $Widgets(citations) deactivate
        }
        get {
            set flavor [lindex $args 0]

            # start with scene citations, then add global citations
            set clist ""
            set sceneObj [$Widgets(scenelist) get selected -data]
            if {$sceneObj != ""} {
                foreach obj [$sceneObj citations get] {
                    lappend clist $obj
                }
            }
            foreach obj $Showcase(citations) {
                lappend clist $obj
            }

            # generate the appropriate flavor for citations
            switch -- $flavor {
                text {
                    set rval ""
                    foreach obj $clist {
                        append rval [$obj produce text] "\n\n"
                    }
                    return $rval
                }
                bibtex {
                    set rval ""
                    foreach obj $clist {
                        append rval [$obj produce bibtex] "\n\n"
                    }
                    return $rval
                }
                endnote {
                    set rval ""
                    foreach obj $clist {
                        append rval [$obj produce endnote] "\n\n"
                    }
                    return $rval
                }
                default {
                    error "bad citation flavor \"$flavor\""
                }
            }
        }
        download {
            main_citations cancel  ;# take down the popup

            set flavor [lindex $args 0]
            switch -- $flavor {
                text    { set fname "citations.txt" }
                bibtex  { set fname "citations.bib" }
                endnote { set fname "citations.enw" }
                default { error "bad citation format \"$flavor\"" }
            }
            Rappture::filexfer::download [main_citations get $flavor] $fname
        }
        default {
            error "bad option \"$option\": should be popup, get, download, cancel"
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_upload_file_select
#
# Called whenever the selection changes in the upload Filebox widget.
# Looks at the current selection and changes the info displayed for
# the current file.
# ----------------------------------------------------------------------
proc main_upload_file_select {} {
    global Widgets
    set fbox $Widgets(upload-files)
    set details $Widgets(upload-detail)

    # clear the contents of the "details" area
    foreach win [winfo children $details] {
        pack forget $win
    }
    foreach win [winfo children $details.info.cntls] {
        pack forget $win
    }

    set fname [$fbox curselection]
    if {$fname ne ""} {
        $details.info.text configure -state normal
        $details.info.text delete 1.0 end
        if {[file isdirectory $fname]} {
            set flist [glob -nocomplain [file join $fname *]]
            set mesg "\tDIRECTORY:\t$fname\n\tCONTAINS:\t[llength $flist] item(s)"
            pack $details.info.cntls.dir -expand yes -fill both
        } else {
            set size "0"
            set bytes [file size $fname]
            foreach {size units} {
                1073741824 GB
                1048576 MB
                1024 kB
                1 B
            } {
                if {$bytes > $size} {
                    set num [expr {$bytes/double($size)}]
                    set size [format {%.1f %s} $num $units]
                    break
                }
            }
            set mesg "\tFILE:\t$fname\n\tSIZE:\t$size\n\tMODIFIED:\t[clock format [file mtime $fname]]"

            if {[file extension $fname] eq ".sho"} {
                pack $details.info.cntls.sho -expand yes -fill both
            } else {
                pack $details.info.cntls.file -expand yes -fill both
            }
        }
        $details.info.text insert 1.0 $mesg
        $details.info.text configure -state disabled
        pack $details.info -fill x -anchor n -padx 20 -pady 20
    }
}

# ----------------------------------------------------------------------
# USAGE: main_upload_file
#
# Called when the user presses "Upload" for a directory.  Prompts the
# user to upload a file and then uploads the file into the directory.
# ----------------------------------------------------------------------
set Upload(jobcntl) ""
set Upload(out) ""

proc main_upload_file {} {
    global Widgets Upload

    set details $Widgets(upload-detail)
    set fbox $Widgets(upload-files)
    set fname [$fbox curselection]

    # is there already an upload running? then kill it
    if {$Upload(jobcntl) ne ""} {
        set Upload(jobcntl) "abort"
        return
    }
    set Upload(jobcntl) "running"
    set Upload(out) ""

    # generate the name of a temp file
    set n 0
    while {[file exists /tmp/mdshowcase$n]} {
        incr n
    }
    set cmd "importfile -p /tmp/mdshowcase$n"

    set status [catch {
        eval blt::bgexec Upload(jobcntl) -output Upload(out) $cmd
    } result]
    set Upload(out) [string trim $Upload(out)]

    if {[regexp {^EXITED} $Upload(jobcntl)]} {
        if {$status == 0
              && [regexp {(.+) +<= +(.+)} $Upload(out) match tmp orig]} {
            set final [file join $fname $orig]
            if {[catch {file rename -force $tmp $final} result] == 0} {
		# Must have world-read permission for render server to
		# read the file.
		file attributes $final -permissions 0664 
                set n [$fbox insert $final]
                $fbox selection clearall
                $fbox selection set $n
            } else {
                file delete -force $tmp
                tk_messageBox -title "Upload: Error" -icon error -message "Oops! Couldn't upload to $final\n$result"
            }
        } else {
            tk_messageBox -title "Upload: Error" -icon error -message "Oops! Upload failed:\n$result"
        }
    }

    # ready for next upload
    set Upload(jobcntl) ""
    set Upload(out) ""
}

# ----------------------------------------------------------------------
# USAGE: main_upload_download
#
# Called when the user presses "Download" for a file.  Kicks off the
# "exportfile" process to download the file to the user's desktop.
# ----------------------------------------------------------------------
proc main_upload_download {} {
    global Widgets
    set fbox $Widgets(upload-files)
    set fname [$fbox curselection]

    if {$fname ne ""} {
        # use exportfile like this to download file in case the file is huge
        catch {exec exportfile $fname &}
    }
}

# ----------------------------------------------------------------------
# USAGE: main_upload_use
#
# Called whenever the user selects a ".sho" file.  Loads that file
# in as the current configuration.
# ----------------------------------------------------------------------
proc main_upload_use {} {
    global Widgets
    set fbox $Widgets(upload-files)
    set fname [$fbox curselection]

    if {[file extension $fname] eq ".sho"} {
        .main current main
        after 1000 [list main_load $fname]
    }
}

# ----------------------------------------------------------------------
# USAGE: main_upload_delete
#
# Called when the user presses "Delete" for a file or directory.  This
# routine is called after the user says they are sure.  It deletes the
# file from the file system and also from the file browser box.
# ----------------------------------------------------------------------
proc main_upload_delete {} {
    global Widgets

    # take down confirmation box
    set details $Widgets(upload-detail)
    $details.info.cntls.popup deactivate

    set fbox $Widgets(upload-files)
    set fname [$fbox curselection]

    if {$fname eq [$fbox cget -fileroot]} {
        # don't delete the main folder, but everything within
        foreach subf [glob -nocomplain [file join $fname *]] {
            catch {file delete -force $subf}
        }
        $fbox configure -fileroot $fname
        $fbox selection clearall
        $fbox selection set 0
    } elseif {$fname ne ""} {
        # delete the specified file
        if {[catch {file delete -force $fname} result] == 0} {
            $fbox delete $fname

            # nothing selected -- clear the contents of the "details" area
            foreach win [winfo children $details] {
                pack forget $win
            }
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_upload_subdirectory
#
# Called whenever the selection changes in the upload Filebox widget.
# Looks at the current selection and changes the info displayed for
# the current file.
# ----------------------------------------------------------------------
proc main_upload_subdirectory {} {
    global Widgets

    # take down confirmation box
    set details $Widgets(upload-detail)
    $details.info.cntls.subdir deactivate

    set fbox $Widgets(upload-files)
    set fname [$fbox curselection]

    # get the current subdir name
    set inner [$details.info.cntls.subdir component inner]
    set subdir [string trim [$inner.name get]]

    # strip strange characters from the subdir name
    regsub -all {[^a-zA-Z0-9_-]} $subdir "" subdir
    set subdir [file join $fname $subdir]

    if {$subdir ne ""} {
        if {[catch {file mkdir $subdir} result] == 0} {
	    file attributes $subdir -permissions 02775
            set n [$fbox insert $subdir]
            $fbox selection clearall
            $fbox selection set $n
        } else {
            tk_messageBox -title "Subdirectory: Error" -icon error -message "Oops! Couldn't create directory:\n$result"
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_download popup
# USAGE: main_download receive <image>
# USAGE: main_download choose image|data|all
# USAGE: main_download download image|data|all
# USAGE: main_download cleanup
#
# Handles download operations.  The "options" argument causes the
# download dialog to appear.  The other options trigger various
# download operations.
# ----------------------------------------------------------------------
proc main_download {option args} {
    global Widgets DownloadOpts MainStatus

    switch -- $option {
        popup {
            $Widgets(viewer) play pause
            set DownloadOpts(file) ""
            $Widgets(dl-go) configure -state disabled
            $Widgets(download) activate $Widgets(main).nav.dl right

            # select the "image" option by default
            set DownloadOpts(which) "image"
            main_download choose image

            # capture the current view if it's not the default
            $Widgets(viewer) scene lastview -prep
        }
        receive {
            if {[llength $args] != 2} {
                error "wrong # args: should be \"main_download receive num image\""
            }
            set DownloadOpts(frame) [lindex $args 0]
            set DownloadOpts(image) [lindex $args 1]

            # show a preview of the image
            set imh [$Widgets(dl-preview-image) cget -image]
            blt::winop resample $DownloadOpts(image) $imh
        }
        choose {
            set what [lindex $args 0]
            foreach win [pack slaves $Widgets(dl-preview)] {
                pack forget $win
            }
            pack $Widgets(dl-preview-$what) -expand yes -fill both -padx 8

            # not ready to download anything yet
            $Widgets(dl-go) configure -state disabled

            # if we're waiting on main_bundle, this will kill it
            main_download cleanup

            switch --  $what {
                image {
                    $Widgets(viewer) getimages frame current \
                        "main_download receive" \
                        "$Widgets(dl-go) configure -state normal"
                    return
                }
                data - all {
                    if {$what eq "data"} {
                        set sceneObj [$Widgets(scenelist) get selected -data]
                        set flist $Widgets(dl-preview-datalist)
                    } else {
                        set sceneObj "all"
                        set flist $Widgets(dl-preview-alllist)
                    }

                    set title "Zip Archive"
                    set fmt "-zip"

                    $flist configure -state normal
                    $flist delete 1.0 end

                    main_spinner start "Building archive file..."
                    foreach {fname manifest} [main_bundle $sceneObj $fmt] break
                    main_spinner stop

                    # load the manifest into the text area
                    if {$manifest ne "--"} {
                        foreach rec $manifest {
                            array set d $rec
                            set d(size) [Rappture::utils::binsize $d(size)]
                            $flist insert end "$d(name) ($d(size))\n"
                        }
                    }
                    $flist configure -state disabled

                    # show the size of the file that will be downloaded
                    if {[file exists $fname]} {
                        set size [Rappture::utils::binsize [file size $fname]]
                        set MainStatus(message) "$title\n$size"

                        set DownloadOpts(file) $fname
                        $Widgets(dl-go) configure -state normal
                    }
                }
                default {
                    error "bad option \"$what\": should be image, data, all"
                }
            }
        }
        download {
            # Shift the file parameter so the file isn't deleted
            # when the download panel is deactivated.  We need to
            # keep it around for a while for the download operation.
            set DownloadOpts(bundle) $DownloadOpts(file)
            set DownloadOpts(file) ""

            $Widgets(download) deactivate

            switch -- [lindex $args 0] {
                image {
                    set fname "frame$DownloadOpts(frame).jpg"
                    set info [$DownloadOpts(image) data -format JPEG]
                    set info [base64::decode $info]
                    Rappture::filexfer::download $info [file tail $fname]
                }
                data - all {
                    if {$DownloadOpts(bundle) ne ""} {
                        # Reach into filexfer and exec the command directly
                        # That avoids having to copy a huge file and bypasses
                        # the download timeout
                        if {[catch {eval blt::bgexec MainStatus(kill) $Rappture::filexfer::commands(export) --delete $DownloadOpts(bundle)} result] != 0} {
                            tk_messageBox -title "Download Error" -icon error -message "Download failed: $result"
                        }

                        main_download cleanup
                    } else {
                        tk_messageBox -title "Download Error" -icon error -message "Oops! Couldn't seem to build archive file for download."
                    }
                }
                default {
                    error "bad option \"[lindex $args 0]\": should be image, data, all"
                }
            }
        }
        cleanup {
            # make sure that any main_bundle operation is killed
            set MainStatus(kill) "abort"

            # if we've built an archive, clean it up
            if {$DownloadOpts(file) ne ""} {
                set dir [file dirname $DownloadOpts(file)]
                file delete -force -- $dir
                set DownloadOpts(file) ""
            }
        }
        default {
            error "bad option \"$option\": should be popup, choose, receive, download, cleanup"
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: main_bundle <sceneObj>|all ?-zip|-tgz?
#
# Used in download operations to create a zip or tarball bundle of
# data for a particular scene object or all scenes.  Returns a list
# with two elements--the archive file and a manifest of contents:
#   archiveFile {{name file1 size 123} {name file2 size 456} ...}
# ----------------------------------------------------------------------
proc main_bundle {what {fmt "-zip"}} {
    global Widgets Showcase MainStatus ToolDir

    # find a temporary directory to build a bundle
    set n 0
    while {[file exists [set tmpdir "/tmp/mdshowcase$n"]]} {
        incr n
    }

    set project [file rootname [file tail $Showcase(file)]]
    if {$project eq ""} { set project "mdshowcase" }
    set bundledir [file join $tmpdir $project]
    file mkdir $bundledir

    # load the "runme" template file -- need this later
    set fid [open [file join $ToolDir runme.tcl] r]
    set runmeScript [read $fid]
    close $fid

    # create internal structure for this bundle
    if {$what eq "all"} {
        set what $Showcase(scenes)
    }

    set manifest ""
    foreach obj $what {
        set n [expr {[lsearch $Showcase(scenes) $obj]+1}]
        set scene "scene$n"
        set flist ""

        # link in all of the data files
        foreach fname [$obj cget -files] {
            if {![info exists linked($fname)]} {
                file link [file join $bundledir [file tail $fname]] $fname
                set linked($fname) yes
                lappend manifest [list name [file tail $fname] \
                    size [file size $fname]]
            }
            lappend flist [file tail $fname]
        }

        # build a file for each scene with citation text
        set citelist ""
        foreach cite [$obj citations get] {
            lappend citelist $cite
        }
        foreach cite $Showcase(citations) {
            lappend citelist $cite
        }

        set citeinfo ""
        foreach cite $citelist {
            append citeinfo [$cite produce text] "\n\n"
        }
        set citeinfo [string trimright $citeinfo]

        set cfile [file join $bundledir "$scene.txt"]
        set fid [open $cfile w]
        puts $fid $citeinfo
        close $fid

        lappend manifest [list name [file tail $cfile] \
            size [file size $cfile]]

        # build a script for each scene
        set sfile [file join $bundledir "$scene.vmd"]
        set fid [open $sfile w]
        puts $fid "#"
        puts $fid "# SCENE $n: [$obj cget -name]"
        if {[$obj cget -description] ne ""} {
            puts $fid "# [$obj cget -description]"
        }
        puts $fid "#"

        puts $fid "# CITE THIS WORK AS FOLLOWS:"
        foreach line [split $citeinfo \n] {
            puts $fid "#  $line"
        }
        puts $fid "#"

        # add code to load the files
        puts $fid [format {
    set op "new"
    foreach fname %s {
        mol $op $fname waitfor all
        set op "addfile"
    }
    foreach nmol [molinfo list] {
        set max [molinfo $nmol get numreps]
        for {set nrep 0} {$nrep < $max} {incr nrep} {
            mol delrep $nrep top
        }
    }
} [list $flist]]

        # this adds the basic script for the view
        puts $fid [$obj cget -vmdscript]

        # if there's a specific last view, save it here
        set info [$Widgets(viewer) scene lastview [$obj cget -name]]
        if {$info ne ""} {
            array set lastview $info
            puts $fid [format {
    # jump to the last view that was on screen
    molinfo top set rotate_matrix {{%s}}
    molinfo top set scale_matrix {{%s}}
    molinfo top set center_matrix {{%s}}
    molinfo top set global_matrix {{%s}}
    animate goto %s
} $lastview(-rotate) $lastview(-scale) $lastview(-center) $lastview(-global) $lastview(-frame)]
        } else {
            set info [$Widgets(viewer) scene initialview [$obj cget -name]]
            array set initview $info
            if {[info exists initview(-frame)] && $initview(-frame) ne ""} {
                puts $fid "    animate goto $initview(-frame)"
            }
        }

        close $fid

        lappend manifest [list name [file tail $sfile] \
            size [file size $sfile]]

        append runmeScript [list choices_add $scene [$obj cget -name] [$obj cget -description]] "\n"
    }

    # write out the final RUNME.tcl script for this example
    set rfile [file join $bundledir RUNME.tcl]
    set fid [open $rfile w]
    puts $fid $runmeScript
    close $fid

    lappend manifest [list name [file tail $rfile] \
        size [file size $rfile]]

    #
    # Build an archive file based on the requested type
    #
    set archive ""
    switch -- $fmt {
        -zip {
            set archive [file join $tmpdir $project.zip]
            set cmd [list zip -r $archive $bundledir]
        }
        -tgz {
            set archive [file join $tmpdir $project.tgz]
            set cmd [list tar cvzf $archive $bundledir]
        }
        default {
            error "bad option \"$fmt\": should be -tgz or -zip"
        }
    }

    if {[catch {eval blt::bgexec MainStatus(kill) $cmd} result]
          || $MainStatus(kill) eq "abort"} {
        file delete -force $tmpdir
        return "-- --"
    }

    return [list $archive $manifest]
}

# ----------------------------------------------------------------------
# USAGE: main_spinner start ?<message>?
# USAGE: main_spinner next <num>
# USAGE: main_spinner stop
#
# Used to update an image with a "busy" spinner.  If the <message> is
# given when the spinner starts, then that message is stored in a
# variable that is displayed in labels showing status.
# ----------------------------------------------------------------------
set MainStatus(message) ""
set MainStatus(afterid) ""
image create photo MainSpinner

proc main_spinner {option {val ""}} {
    global MainStatus

    switch -- $option {
        start {
            main_spinner stop
            main_spinner next 1

            if {$val eq ""} { set val "Busy..." }
            set MainStatus(message) $val
        }
        next {
            MainSpinner blank
            MainSpinner copy [Rappture::icon roller$val]
            if {[incr val] > 8} { set val 1 }
            set MainStatus(afterid) [after 200 main_spinner next $val]
        }
        stop {
            if {$MainStatus(afterid) ne ""} {
                after cancel $MainStatus(afterid)
                set MainStatus(afterid) ""
            }
            MainSpinner blank
            set MainStatus(message) ""
        }
        default {
            error "bad option \"$option\": should be start, stop, next"
        }
    }
}

# ----------------------------------------------------------------------
# MAIN WINDOW
# ----------------------------------------------------------------------
BackForth .main
pack .main -expand yes -fill both

set main [.main insert end main]
set Widgets(main) $main

set options [.main insert end options]
.main position options -leftof main
set Widgets(options) $options

set upload [.main insert end upload]
.main position upload -below main
set Widgets(upload) $upload

Rappture::Postern $main.postern
pack $main.postern -side bottom -fill x

frame $main.nav
pack $main.nav -side left -fill y -pady 20

set citations [Rappture::Balloon .main.citations \
    -title "Cite this work as follows"]
set Widgets(citations) $citations

button $main.nav.cite -compound left -padx 4 \
    -text "Cite this work..." -image [Rappture::icon quill] \
    -command {main_citations popup}
pack $main.nav.cite -anchor w -padx 8 -pady 0

set download [Rappture::Balloon .main.dlopts -title "Download" \
    -deactivatecommand {main_download cleanup}]
set Widgets(download) $download

button $main.nav.dl -compound left -padx 4 \
    -text "Download data for this view..." -image [Rappture::icon download] \
    -command {main_download popup}
pack $main.nav.dl -anchor w -padx 8 -pady {0 8}

Controlbox $main.nav.list -selectcommand main_scene_change
pack $main.nav.list -fill x -padx {6 0}
set Widgets(scenelist) $main.nav.list

button $main.nav.opts -text "More controls..." -compound left \
    -image "" -command {.main current options}
pack $main.nav.opts -side top -anchor w -padx 6 -pady {8 0}

if {$Editor} {
    button $main.nav.upload -text "Upload trajectories and scripts..." \
        -compound left -image [Rappture::icon folder2] \
        -command {main_scene_reset; .main current upload}
    pack $main.nav.upload -side bottom -fill x -padx 6 -pady 2
}

frame $main.view
pack $main.view -expand yes -fill both -padx {0 8} -pady 8
frame $main.view.cntl
pack $main.view.cntl -side bottom -fill x
Rappture::VMDvisViewer $main.view.vmd \
    -changecommand main_scene_changes \
    -waitingcommand "main_scene_changes waiting"
pack $main.view.vmd -expand yes -fill both
set Widgets(viewer) $main.view.vmd

label $main.view.cntl.reset -image [Rappture::icon vreset]
pack $main.view.cntl.reset -side left -padx {0 8}
bind $main.view.cntl.reset <ButtonPress> {main_scene_reset}
Rappture::Tooltip::for $main.view.cntl.reset "Reset the view back to the default.  This usually centers the molecule and resets the zoom level."

label $main.view.cntl.rotate -image [Rappture::icon rotate0]
pack $main.view.cntl.rotate -side left
bind $main.view.cntl.rotate <ButtonPress> {main_scene_mode rotate}
Rappture::Tooltip::for $main.view.cntl.rotate "Click to enter \"rotate\" mode.  Then click and drag on the screen to rotate the view."

label $main.view.cntl.pan -image [Rappture::icon pan0]
pack $main.view.cntl.pan -side left
bind $main.view.cntl.pan <ButtonPress> {main_scene_mode pan}
Rappture::Tooltip::for $main.view.cntl.pan "Click to enter \"pan\" mode.  Then click and drag on the screen to shift the view."

label $main.view.cntl.zoom -image [Rappture::icon zoom0]
pack $main.view.cntl.zoom -side left
bind $main.view.cntl.zoom <ButtonPress> {main_scene_mode zoom}
Rappture::Tooltip::for $main.view.cntl.zoom "Click to enter \"zoom\" mode.  Then click and drag up/down on the screen to zoom in/out."

VCR $main.view.cntl.vcr -command main_scene_frames
pack $main.view.cntl.vcr -side left -expand yes -fill both -padx 8
set Widgets(VCR) $main.view.cntl.vcr

after idle [list main_scene_mode rotate]

# ----------------------------------------------------------------------
# OPTIONS WINDOW
# ----------------------------------------------------------------------
frame $options.cntl
pack $options.cntl -side top -fill x
frame $options.cntl.line -height 1 -background black
pack $options.cntl.line -side bottom -fill x
button $options.cntl.back -compound right -padx 4 \
    -text "Back" -image [Rappture::icon arrow-rt-black] \
    -command {.main current main}
pack $options.cntl.back -side right

# ----------------------------------------------------------------------
# UPLOAD WINDOW
# ----------------------------------------------------------------------
frame $upload.cntl
pack $upload.cntl -side top -fill x
frame $upload.cntl.line -height 1 -background black
pack $upload.cntl.line -side bottom -fill x
button $upload.cntl.back -compound right -padx 4 \
    -text "Back" -image [Rappture::icon arrow-up-black] \
    -command {.main current main}
pack $upload.cntl.back -side right

frame $upload.upload
pack $upload.upload -expand yes -fill both -padx 8 -pady 8
Rappture::Panes $upload.upload.pw -orientation horizontal \
    -sashcursor sb_h_double_arrow
pack $upload.upload.pw -expand yes -fill both
$upload.upload.pw fraction 0 0.25

set win [$upload.upload.pw pane 0]
label $win.l -text "Shared files:"
pack $win.l -side top -anchor w
Rappture::Scroller $win.scrl -xscrollmode auto -yscrollmode auto
pack $win.scrl -expand yes -fill both
Filebox $win.scrl.files -fileroot $DataFolder -hideroot no \
    -selectcommand main_upload_file_select
$win.scrl contents $win.scrl.files
set Widgets(upload-files) $win.scrl.files

set win [$upload.upload.pw insert end]
set Widgets(upload-detail) $win

frame $win.info
set Widgets(upload-info) $win.info
text $win.info.text -height 3 -highlightthickness 0 -wrap none
pack $win.info.text -expand yes -fill both
$win.info.text configure -tabs "1i right 1.1i left"
frame $win.info.cntls
pack $win.info.cntls -fill x

Rappture::Balloon $win.info.cntls.subdir -title "Add Subdirectory"
set inner [$win.info.cntls.subdir component inner]
label $inner.l -text "Name:"
pack $inner.l -side left
entry $inner.name -width 30
pack $inner.name -side left
bind $inner.name <KeyPress-Return> main_upload_subdirectory

Rappture::Balloon $win.info.cntls.popup -title "Confirm Delete"
set inner [$win.info.cntls.popup component inner]
label $inner.mesg -text "Are you really sure you want to delete?\nThis can't be undone."
pack $inner.mesg -fill x -padx 8 -pady 20
button $inner.ok -width 14 -text "Yes, Delete" -command main_upload_delete
pack $inner.ok -side left -expand yes -padx 8 -pady 4
button $inner.cancel -width 14 -text "No, Cancel" -command [list $win.info.cntls.popup deactivate]
pack $inner.cancel -side left -expand yes -padx 8 -pady 4

# use these controls for directories
frame $win.info.cntls.dir
button $win.info.cntls.dir.upl -text "Upload File" -command main_upload_file
pack $win.info.cntls.dir.upl -side left -padx 8 -pady 8
button $win.info.cntls.dir.subd -text "Add Subdirectory" -command [list $win.info.cntls.subdir activate $win.info.cntls.dir.subd below]
pack $win.info.cntls.dir.subd -side left -padx 8 -pady 8
button $win.info.cntls.dir.del -text "Delete Entire Directory" -command [list $win.info.cntls.popup activate $win.info.cntls.dir.del below]
pack $win.info.cntls.dir.del -side left -padx 8 -pady 8

# use these controls for .sho files
frame $win.info.cntls.sho
button $win.info.cntls.sho.use -text "Load File" -command main_upload_use
pack $win.info.cntls.sho.use -side left -padx 8 -pady 8
#button $win.info.cntls.sho.edit -text "Edit File" -command main_upload_editor_open
#pack $win.info.cntls.sho.edit -side left -padx 8 -pady 8
button $win.info.cntls.sho.dnl -text "Download File" -command main_upload_download
pack $win.info.cntls.sho.dnl -side left -padx 8 -pady 8
button $win.info.cntls.sho.del -text "Delete File" -command [list $win.info.cntls.popup activate $win.info.cntls.sho.del below]
pack $win.info.cntls.sho.del -side left -padx 8 -pady 8

# use these controls for all other files
frame $win.info.cntls.file
button $win.info.cntls.file.dnl -text "Download File" -command main_upload_download
pack $win.info.cntls.file.dnl -side left -padx 8 -pady 8
button $win.info.cntls.file.del -text "Delete File" -command [list $win.info.cntls.popup activate $win.info.cntls.file.del below]
pack $win.info.cntls.file.del -side left -padx 8 -pady 8

# editor area for .show files
frame $win.editor
set Widgets(upload-editor) $win.editor
frame $win.editor.cntls
pack $win.editor.cntls -fill x
button $win.editor.cntls.save -text "Save" -command main_upload_editor_save
pack $win.editor.cntls.save -side left -padx 8 -pady 4
button $win.editor.cntls.cancel -text "Cancel" -command main_upload_editor_cancel
pack $win.editor.cntls.cancel -side right -padx 8 -pady 4
Rappture::Scroller $win.editor.scrl -xscrollmode auto -yscrollmode auto
pack $win.editor.scrl -expand yes -fill both
text $win.editor.scrl.text
$win.editor.scrl contents $win.editor.scrl.text

# ----------------------------------------------------------------------
# CITATIONS WINDOW
# ----------------------------------------------------------------------
set f [$citations component inner]
frame $f.download
pack $f.download -side bottom -fill x

label $f.download.l -text "Download citation info as:"
pack $f.download.l -side left -padx {8 0}

button $f.download.text -compound left -padx 4 \
    -text "Text" -image [Rappture::icon download] \
    -command {main_citations download text}
pack $f.download.text -side left -padx 2 -pady 8

button $f.download.bibtex -compound left -padx 4 \
    -text "BibTex" -image [Rappture::icon download] \
    -command {main_citations download bibtex}
pack $f.download.bibtex -side left -padx 2 -pady 8

button $f.download.endnote -compound left -padx 4 \
    -text "EndNote" -image [Rappture::icon download] \
    -command {main_citations download endnote}
pack $f.download.endnote -side left -padx 2 -pady 8

Rappture::Scroller $f.scrl
pack $f.scrl -expand yes -fill both -padx 20 -pady 4
text $f.scrl.text
$f.scrl contents $f.scrl.text
set Widgets(citations-info) $f.scrl.text

# ----------------------------------------------------------------------
# DOWNLOAD WINDOW
# ----------------------------------------------------------------------
set f [$download component inner]

frame $f.which
pack $f.which -anchor w -padx 8 -pady 4
radiobutton $f.which.im -text "Download current image" \
    -variable DownloadOpts(which) -value image \
    -command {main_download choose image}
pack $f.which.im -anchor w

radiobutton $f.which.data -text "Download data for the current scene" \
    -variable DownloadOpts(which) -value data \
    -command {main_download choose data}
pack $f.which.data -anchor w

radiobutton $f.which.all -text "Download data for all scenes" \
    -variable DownloadOpts(which) -value all \
    -command {main_download choose all}
pack $f.which.all -anchor w

frame $f.preview -width 4i -height 200
pack propagate $f.preview off
pack $f.preview -expand yes -fill both -padx 8 -pady 4
set Widgets(dl-preview) $f.preview

label $f.preview.im -image [image create photo -width 200 -height 200]
pack $f.preview.im
set Widgets(dl-preview-image) $f.preview.im

frame $f.preview.dlone
set Widgets(dl-preview-data) $f.preview.dlone
frame $f.preview.dlone.top
pack $f.preview.dlone.top -fill x
label $f.preview.dlone.top.im -image [Rappture::icon zipfile64]
pack $f.preview.dlone.top.im -side left
label $f.preview.dlone.top.status -textvariable MainStatus(message) -justify left
pack $f.preview.dlone.top.status -anchor w
label $f.preview.dlone.top.spinner -image MainSpinner
pack $f.preview.dlone.top.spinner -anchor w
Rappture::Scroller $f.preview.dlone.info -xscrollmode none -yscrollmode auto -width 1i -height 1i
pack $f.preview.dlone.info -expand yes -fill both
text $f.preview.dlone.info.txt -borderwidth 0 -relief flat -wrap none -state disabled
$f.preview.dlone.info contents $f.preview.dlone.info.txt
set Widgets(dl-preview-datalist) $f.preview.dlone.info.txt

frame $f.preview.dlall
set Widgets(dl-preview-all) $f.preview.dlall
frame $f.preview.dlall.top
pack $f.preview.dlall.top -fill x
label $f.preview.dlall.top.im -image [Rappture::icon zipfile64]
pack $f.preview.dlall.top.im -side left
label $f.preview.dlall.top.status -textvariable MainStatus(message) -justify left
pack $f.preview.dlall.top.status -anchor w
label $f.preview.dlall.top.spinner -image MainSpinner
pack $f.preview.dlall.top.spinner -anchor w
Rappture::Scroller $f.preview.dlall.info -xscrollmode none -yscrollmode auto -width 1i -height 1i
pack $f.preview.dlall.info -expand yes -fill both
text $f.preview.dlall.info.txt -borderwidth 0 -relief flat -wrap none -state disabled
$f.preview.dlall.info contents $f.preview.dlall.info.txt
set Widgets(dl-preview-alllist) $f.preview.dlall.info.txt

frame $f.cntls
pack $f.cntls -side bottom -padx 8 -pady 8
button $f.cntls.go -width 8 -text "Download" \
    -command {main_download download $DownloadOpts(which)}
pack $f.cntls.go -side left -padx 8
set Widgets(dl-go) $f.cntls.go

button $f.cntls.cancel -width 8 -text "Cancel" \
    -command [list $Widgets(download) deactivate]
pack $f.cntls.cancel -side left -padx 8


# ----------------------------------------------------------------------
# Look for an input file and load the showcase
# ----------------------------------------------------------------------
set file ""
set scenenum ""

if {[info exists env(TOOL_PARAMETERS)]
      && [file readable $env(TOOL_PARAMETERS)]} {
    set fid [open $env(TOOL_PARAMETERS) r]
    set info [read $fid]
    close $fid

    foreach line [split $info \n] {
        set line [string trim $line]
        if {$line eq "" || [regexp {^#} $line]} {
            continue
        }

        if {[regexp {^([a-zA-Z]+)(\([^)]+\))?\:(.*)} $line match type name value]} {
            if {$type eq "file" && $file eq ""} {
                set file [string trim $value]
            } elseif {$type eq "int" && $scenenum eq ""} {
                set scenenum [string trim $value]
            }
        } else {
            puts stderr "ERROR: bad line in TOOL_PARAMETERS file"
            puts stderr ">>$line"
            puts stderr "skipping..."
            break
        }
    }
}

# if there's a file on the command line, then it overrides everything
if {[llength $argv] == 1} {
    set str [lindex $argv 0]
    if {[file extension $str] eq ".sho" && [file readable $str]} {
        set file $str
    }
}

# last resort -- try to use the built-in input file
if {![file readable $file]} {
    set dir [file dirname [file dirname [info script]]]
    set file [file join $dir examples test.sho]
}

# couldn't get a valid input file? then fail
if {[file readable $file]} {
    main_load $file $scenenum
} else {
    tk_messageBox -icon error -message "No input file, and can't find example file in /apps"
}

# make sure that main window keeps focus for keyboard shortcuts
bind . <ButtonPress> {focus .}
bind [$Widgets(viewer) component 3dview] <Enter> {focus .}

after 10 {focus .}
