# ----------------------------------------------------------------------
#  SCRIPT: RUNME.tcl
#
#  Load this script into VMD to reproduce the experience that you see
#  in the Molecular Dynamics Showcase on nanoHUB.org
# ======================================================================
#  Copyright (c) 2013  Purdue University
# ======================================================================
toplevel .main
wm title .main "MD Showcase"

option add *heading.font {Helvetica -12 italic} startupFile
option add *nav.titleFont {Helvetica -12 bold} widgetDefault
option add *nav.subtitleFont {Helvetica -12} widgetDefault
option add *nav.padding 4 widgetDefault
option add *nav.lineColor #999 widgetDefault
option add *nav.entryForeground white widgetDefault
option add *nav.entryBackground #666 widgetDefault
option add *nav.selectedEntryForeground black widgetDefault
option add *nav.selectedEntryBackground white widgetDefault
option add *nav.hiliteEntryForeground black widgetDefault
option add *nav.hiliteEntryBackground #ffffcc widgetDefault
option add *nav.arrowColor black widgetDefault
option add *Text.borderWidth 0 startupFile
option add *Text.background white startupFile
option add *Text.font {Courier -12} startupFile
option add *Scrollbar.borderWidth 1 startupFile
option add *Scrollbar.relief flat startupFile
option add *Scrollbar.troughColor #999 startupFile

set InstallDir [file dirname [info script]]

# ----------------------------------------------------------------------
# CAPTURE STARTING DISPLAY CHARACTERISTICS
# ----------------------------------------------------------------------
display antialias on

set DisplayProps(options) ""
foreach key {
    ambientocclusion antialias aoambient aodirect
    backgroundgradient
    culling cuestart cueend cuedensity cuemode
    depthcue distance
    eyesep
    farclip focallength
    height
    nearclip
    projection
    shadows stereo
} {
    if {$key eq "nearclip" || $key eq "farclip"} {
        append DisplayProps(options) [list display $key set [display get $key]] "\n"
    } else {
        append DisplayProps(options) [list display $key [display get $key]] "\n"
    }
}

# ----------------------------------------------------------------------
# MAIN WINDOW
# ----------------------------------------------------------------------
image create photo quill -data {
R0lGODlhEgASAMQAABeX2S5PYMnJyUOJrDNqhprC17a3uIqKig4ODi6c1F6pzxeFvnGy02BxeXV6
fNTU1FhocKmqqoq71WuOoEtPUczV2Rx4qL3O2Co2PBeNyD9gcCNvl5mbnCZ9qhWQz9nZ2SH/C1hN
UCBEYXRhWE1QPD94cGFja2V0IGJlZ2luPSLvu78iIGlkPSJXNU0wTXBDZWhpSHpyZVN6TlRjemtj
OWQiPz4gPHg6eG1wbWV0YSB4bWxuczp4PSJhZG9iZTpuczptZXRhLyIgeDp4bXB0az0iQWRvYmUg
WE1QIENvcmUgNS4wLWMwNjEgNjQuMTQwOTQ5LCAyMDEwLzEyLzA3LTEwOjU3OjAxICAgICAgICAi
PiA8cmRmOlJERiB4bWxuczpyZGY9Imh0dHA6Ly93d3cudzMub3JnLzE5OTkvMDIvMjItcmRmLXN5
bnRheC1ucyMiPiA8cmRmOkRlc2NyaXB0aW9uIHJkZjphYm91dD0iIiB4bWxuczp4bXA9Imh0dHA6
Ly9ucy5hZG9iZS5jb20veGFwLzEuMC8iIHhtbG5zOnhtcE1NPSJodHRwOi8vbnMuYWRvYmUuY29t
L3hhcC8xLjAvbW0vIiB4bWxuczpzdFJlZj0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL3NU
eXBlL1Jlc291cmNlUmVmIyIgeG1wOkNyZWF0b3JUb29sPSJBZG9iZSBQaG90b3Nob3AgQ1M1LjEg
TWFjaW50b3NoIiB4bXBNTTpJbnN0YW5jZUlEPSJ4bXAuaWlkOjAzNzIzRTlCOUIzRjExRTNBMUJB
RTAxQTdBQzQyREFGIiB4bXBNTTpEb2N1bWVudElEPSJ4bXAuZGlkOjAzNzIzRTlDOUIzRjExRTNB
MUJBRTAxQTdBQzQyREFGIj4gPHhtcE1NOkRlcml2ZWRGcm9tIHN0UmVmOmluc3RhbmNlSUQ9Inht
cC5paWQ6MDM3MjNFOTk5QjNGMTFFM0ExQkFFMDFBN0FDNDJEQUYiIHN0UmVmOmRvY3VtZW50SUQ9
InhtcC5kaWQ6MDM3MjNFOUE5QjNGMTFFM0ExQkFFMDFBN0FDNDJEQUYiLz4gPC9yZGY6RGVzY3Jp
cHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBhY2tldCBlbmQ9InIiPz4B//79/Pv6
+fj39vX08/Lx8O/u7ezr6uno5+bl5OPi4eDf3t3c29rZ2NfW1dTT0tHQz87NzMvKycjHxsXEw8LB
wL++vby7urm4t7a1tLOysbCvrq2sq6qpqKempaSjoqGgn56dnJuamZiXlpWUk5KRkI+OjYyLiomI
h4aFhIOCgYB/fn18e3p5eHd2dXRzcnFwb25tbGtqaWhnZmVkY2JhYF9eXVxbWllYV1ZVVFNSUVBP
Tk1MS0pJSEdGRURDQkFAPz49PDs6OTg3NjU0MzIxMC8uLSwrKikoJyYlJCMiISAfHh0cGxoZGBcW
FRQTEhEQDw4NDAsKCQgHBgUEAwIBAAAh+QQAAAAAACwAAAAAEgASAAAFjeAnjuR4SYxilGyhJJ4H
OOwowXG80Oyd5wBChMX4xQAAD+RRKiZjmYVloxmSJDECREbgCARM0gWQCVAmSEBisCopFhjEBNYh
EKqlgifeSCA7EwI1CgEIFAoeU1RtJBUDCBgKAAtdBmElF4VoABZ4NSIYGglUB4KfHwcIAhERpqcP
CDynJKmusyIcsrciIQA7}

canvas .main.nav -width 3i -height 1i -borderwidth 0 -highlightthickness 0
pack .main.nav -side left -fill y -padx {12 0} -pady 32
bind .main.nav <Configure> choices_redraw
bind .main.nav <Motion> {choices_event motion %x %y}
bind .main.nav <ButtonPress> {choices_event click %x %y}
bind .main.nav <Leave> {choices_event motion -1 -1}

frame .main.cite
pack .main.cite -expand yes -fill both

label .main.cite.heading -compound left -padx 4 \
    -text "Cite this work as follows:" -image quill
pack .main.cite.heading -anchor w

frame .main.cite.info
pack .main.cite.info -expand yes -fill both -padx {0 8} -pady {0 8}
scrollbar .main.cite.info.ys -orient vertical -highlightthickness 0 \
    -command {.main.cite.info.text yview}
pack .main.cite.info.ys -side right -fill y -padx 2
text .main.cite.info.text -width 60 -height 20 \
    -borderwidth 8 -relief flat -highlightthickness 0 \
    -yscrollcommand {.main.cite.info.ys set}
pack .main.cite.info.text -expand yes -fill both
.main.cite.info.text configure -state disabled

set Choices(widget) .main.nav
set Choices(counter) 0
set Choices(all) ""
set Choices(selected) ""
set Choices(action) load_scene

# ----------------------------------------------------------------------
# USAGE: choices_add <data> <text> <description>
#
# Registers a choice in the choice list used to select views.
# ----------------------------------------------------------------------
proc choices_add {data text desc} {
    global Choices

    set name "choice[incr Choices(counter)]"
    set Choices($name-data) $data
    set Choices($name-text) $text
    set Choices($name-desc) $desc
    lappend Choices(all) $name

    if {[llength $Choices(all)] == 1} {
        set Choices(selected) $name
        after idle $Choices(action) $Choices($name-data)
    }

    after idle choices_redraw
}

# ----------------------------------------------------------------------
# USAGE: choices_redraw
#
# Registers a choice in the choice list used to select views.
# ----------------------------------------------------------------------
proc choices_redraw {} {
    global Choices
    set c $Choices(widget)
    set w [winfo width $c]

    $c delete all
    set Choices(hilite) ""

    set pad [option get $c padding Padding]
    set tfont [option get $c titleFont TitleFont]
    set tht [font metrics $tfont -linespace]
    set subtfont [option get $c subtitleFont Font]
    set subtht [font metrics $subtfont -linespace]
    set lineht [expr {$tht+$subtht+3*$pad}]
    set leftpad [expr {$lineht/2}]

    set fg [option get $c entryForeground Foreground]
    set bg [option get $c entryBackground Background]
    set selfg [option get $c selectedEntryForeground Background]
    set selbg [option get $c selectedEntryBackground Foreground]
    set arrfg [option get $c arrowColor Foreground]
    set linec [option get $c lineColor Foreground]

    # run through the items and build the layout
    set Choices(layout) ""
    set ypos 0
    foreach tid $Choices(all) {
        set xpos [expr {$leftpad+$pad}]
        incr ypos $pad
        set y0 [expr {$ypos-$pad}]
        set y1 [expr {$ypos+$lineht+$pad}]
        set ymid [expr {$ypos+$lineht/2}]
        $c create oval 0 $y0 $lineht $y1 \
            -outline "" -fill "" -tags [list round round-$tid]
        $c create rectangle $leftpad $y0 $w [expr {$y1+1}] \
            -outline "" -fill "" -tags [list bg bg-$tid]

        set tsize [expr {$lineht/2-2*$pad}]
        $c create polygon \
            [expr {$w-$tsize-2*$pad}] [expr {$y0+2*$pad}] \
            [expr {$w-2*$pad}] $ymid \
            [expr {$w-$tsize-2*$pad}] [expr {$y1-2*$pad}] \
            -outline "" -fill "" -tags [list arrow arrow-$tid]

        set text $Choices($tid-text)
        set desc $Choices($tid-desc)
        set xtext [expr {$xpos+2*$pad}]

        if {$text ne "" && $desc ne ""} {
            $c create text $xtext [expr {$ymid-$pad/2}] -anchor sw \
                -text $text -font $tfont \
                -fill $fg -tags [list text text-$tid]
            $c create text $xtext [expr {$ymid+$pad/2}] -anchor nw \
                -text $desc -font $subtfont \
                -fill $fg -tags [list text text-$tid]
        } elseif {$text ne ""} {
            $c create text $xtext $ymid -anchor w \
                -text $text -font $tfont \
                -fill $fg -tags [list text text-$tid]
        } elseif {$desc ne ""} {
            $c create text $text $ymid -anchor w \
                -text $desc -font $subtfont \
                -fill $fg -tags [list text text-$tid]
        }

        if {$tid eq $Choices(selected)} {
            $c itemconfigure bg-$tid -outline "" -fill $selbg
            $c itemconfigure round-$tid -outline "" -fill $selbg
            $c itemconfigure text-$tid -fill $selfg
            $c itemconfigure arrow-$tid -fill $arrfg
        } else {
            $c itemconfigure bg-$tid -outline $linec -fill $bg
            $c itemconfigure text-$tid -fill $fg
        }

        set ypos [expr {$ypos + $lineht + $pad}]
        lappend Choices(layout) $ypos $tid
    }

    # fix the final size so the widget scrolls properly
    set x1 0; set y1 0
    foreach {x0 y0 x1 y1} [$c bbox all] break
    set x1 [expr {$x1+$pad}]
    set y1 [expr {$y1+$pad+1}]
    $c configure -scrollregion [list 0 0 $x1 $y1]

    # synthesize a motion event to pick up a new hilite
    set x [expr {[winfo pointerx $c]-[winfo rootx $c]}]
    set y [expr {[winfo pointery $c]-[winfo rooty $c]}]
    choices_event motion $x $y

    $c configure -height $y1
}

# ----------------------------------------------------------------------
# USAGE: choices_event motion <x> <y>
# USAGE: choices_event click <x> <y>
#
# Handles mouse pointer events when the user interacts with the
# "choices" list.
# ----------------------------------------------------------------------
proc choices_event {option x y} {
    global Choices
    set c $Choices(widget)

    set fg [option get $c entryForeground Foreground]
    set bg [option get $c entryBackground Background]
    set selfg [option get $c selectedEntryForeground Background]
    set selbg [option get $c selectedEntryBackground Foreground]
    set hltfg [option get $c hiliteEntryForeground Foreground]
    set hltbg [option get $c hiliteEntryBackground Background]
    set arrfg [option get $c arrowColor Foreground]
    set linec [option get $c lineColor Foreground]

    set tid ""
    if {$y >= 0} {
        foreach {ypos item} $Choices(layout) {
            if {$y < $ypos} {
                set tid $item
                break
            }
        }
    }

    switch -- $option {
        motion {
            if {$tid ne $Choices(hilite)} {
                if {$Choices(hilite) ne ""} {
                    $c itemconfigure bg-$Choices(hilite) -fill $bg
                    $c itemconfigure text-$Choices(hilite) -fill $fg
                }

                if {$tid ne "" && $tid ne $Choices(selected)} {
                    # turn on hilite color
                    $c itemconfigure bg-$tid -fill $hltbg
                    $c itemconfigure text-$tid -fill $hltfg
                    set Choices(hilite) $tid
                }
            }
        }
        click {
            if {$tid ne "" && $tid ne $Choices(selected)} {
                $c itemconfigure bg -outline $linec -fill $bg
                $c itemconfigure text -fill $fg
                $c itemconfigure round -fill ""
                $c itemconfigure arrow -fill ""

                $c itemconfigure bg-$tid -outline "" -fill $selbg
                $c itemconfigure round-$tid -fill $selbg
                $c itemconfigure text-$tid -fill $selfg
                $c itemconfigure arrow-$tid -fill $arrfg

                $c raise bg-$tid
                $c raise text-$tid
                $c raise arrow-$tid

                if {$Choices(action) ne ""} {
                    after idle $Choices(action) $Choices($tid-data)
                }
            }
            set Choices(selected) $tid
            set Choices(hilite) ""
        }
        default {
            error "don't know how to handle action \"$option\""
        }
    }
}

# ----------------------------------------------------------------------
# USAGE: load_scene <name>
#
# Loads the scene script and citation data from the files with the
# root <name> into the VMD window and the main screen.
# ----------------------------------------------------------------------
proc load_scene {name} {
    global InstallDir DisplayProps

    # load data into the citation window
    set t .main.cite.info.text
    $t configure -state normal
    $t delete 1.0 end

    if {[catch {
      set fid [open [file join $InstallDir $name.txt] r]
      set info [read $fid]
      close $fid
       }] == 0} {
        $t insert end $info
    }
    $t configure -state disabled

    # reset the display options
    display resetview
    axes location off
    color Display Background black
    eval $DisplayProps(options)

    # clear the molecule and load the new one
    foreach nmol [molinfo list] { 
        set max [molinfo $nmol get numreps]
        for {set nrep 0} {$nrep < $max} {incr nrep} { 
            mol delrep 0 top
        }
        mol delete $nmol
    }

    source $name.vmd
}
